/*!
 * Telegraft: broadcast
 * Copyright(c) 2012 jquarry.com
 * MIT Licensed
 */

/**
 * Module dependencies.
 */

var _ = require('underscore'),
		handler = require('./handler');

/**
 * Broadcast prototype.
 */

var broadcast = exports = module.exports = {};

/**
 * Initialize the broadcast.
 *
 *   - setup default configuration
 *
 * @api private
 */

broadcast.init = function(){
  var self = this;
  this._nextActionId = 0;
  this.settings = {};
  this._queue = [];
  this._frequency = [];
  this._middleware = [];
  this._transmissionGroups = [];
  this._currentTransmissionGroup = [];
  this.defaultConfiguration();
};

/**
 * Initialize broadcast configuration.
 *
 * @api private
 */

broadcast.defaultConfiguration = function(){
  var self = this;

};

/**
 * Interface to the current queue
 *
 * @api private
 */

broadcast.queue = function(action){

	// assume they want a certain queue item
	if(_.isNumber(action)){
  	return this._queue[action];
  }

	// validate the queue entry
	if(!_.isObject(action) || _.isEmpty(action.handler) || _.isEmpty(action.data)) {
		return this._queue;	
	}

  action.id = this._nextActionId++;

	this._queue.push(action);

  return this._queue;

};

/**
 * Change the broadcast frequency
 *
 * @api public
 */

broadcast.frequency = function(freq){
  if(_.isEmpty(freq)){
   	return this._frequency;
  }
	if(_.isString(freq)){
		this._frequency=[freq];
	}
	else if(_.isArray(freq)){
		this._frequency=freq;
	}

  this.queue({
    handler:'frequency',
    data:freq
  });
	
  return this._frequency;
};

/**
 * Use some middleware
 *
 * @api public
 */

broadcast.middleware = function(middleware){
	if(!_.isFunction(middleware)){
		return this._middleware;
	}
  
  this._middleware.push(middleware);

  return this._middleware;
	
};

/**
 * Send the broadcast using the transmitter
 *
 * @api public
 */

broadcast.transmit = function(freq){
  this.frequency(freq);

  return this;
};

/**
 * Accept a recording into the current sequence
 *
 * @api public
 */

broadcast.record = function(param, data){

	var self = this;
	
	// we run the function against the broadcast
	if(_.isFunction(param)){
  	param.apply(self, [self]);
  }
	// we iterate and add an announcement for each key and record the value
  else if(_.isObject(param)){
  	_.each(param, function(recording,announcement){
  		self.announce(announcement);
  		self.record(recording);
  	});
  }
  // we iterate and record each value
  else if(_.isArray(param)){
		_.each(param, function(recording){
  		self.record(recording);
  	})
  }
  // we assume param to be either an SSH command or handler + data
  else if(_.isString(param)){
  	// assume SSH as default

  	if(!data){
  		data = param;
  		param = 'ssh';
  	}

  	this.queue({
  		handler:param,
  		data:data
  	});	
  }
  
  return this;

};

/**
 * Accept a wait instruction into the current sequence
 *
 * @api public
 */

broadcast.wait = function(){
  
  this.queue({
    handler:'wait',
    data:'wait'
  });

	return this;
};

